<?php
/**
 * @file
 * Views filter handler to display UI for WHERE clauses and prepare them for
 * the query.
 */

class sparql_views_handler_filter extends views_handler_filter_string{
  // We will use the value type and language form elements for getting
  // necessary info about replacements that this filter makes.
  var $selected_sparql_options = array('value_type', 'language');

  function option_definition() {
    $options = parent::option_definition();
    // Attach the SPARQL specific options.
    $options = _sparql_views_attach_form_options($options, $this->selected_sparql_options);

    return $options;
  }
  
  /**
   * This kind of construct makes it relatively easy for a child class
   * to add or remove functionality by overriding this function and
   * adding/removing items from this array.
   */
  function operators() {
    $operators = array(
      '=' => array(
        'title' => t('Is equal to'),
        'short' => t('='),
        'method' => 'op_equal',
        'values' => 1,
      ),
      '!=' => array(
        'title' => t('Is not equal to'),
        'short' => t('!='),
        'method' => 'op_not_equal',
        'values' => 1,
      ),
      'not_bound' => array(
        'title' => t('Not Bound'),
        'short' => t('!BOUND'),
        'method' => 'op_not_bound',
        'values' => 1,
      ),
      'is_uri' => array(
        'title' => t('Is A URI'),
        'short' => t('isURI'),
        'method' => 'op_is_uri',
        'values' => 0,
      ),
      'datatype' => array(
        'title' => t('Is of data type'),
        'short' => 'DATATYPE()',
        'method' => 'op_is_of_datatype',
        'values' => 1,
      ),
    );
    return $operators;
  }

  function admin_summary() {
    $output = $this->operator . ' ';
    $output .= check_plain($this->options['value']);
    $output .= "<i>({$this->options['sparql_options']['value_type']})</i>";
    return $output;
  }
  
  function value_form(&$form, &$form_state) {
    // We have to make some choices when creating this as an exposed
    // filter form. For example, if the operator is locked and thus
    // not rendered, we can't render dependencies; instead we only
    // render the form items we need.
    $which = 'all';
    if (!empty($form['operator'])) {
      $source = ($form['operator']['#type'] == 'radios') ? 'radio:options[operator]' : 'edit-options-operator';
    }
    if (!empty($form_state['exposed'])) {
      $identifier = $this->options['expose']['identifier'];

      if (empty($this->options['expose']['use_operator']) || empty($this->options['expose']['operator'])) {
        // exposed and locked.
        $which = in_array($this->operator, $this->operator_values(1)) ? 'value' : 'none';
      }
      else {
        $source = 'edit-' . drupal_html_id($this->options['expose']['operator']);
      }
    }

    if ($which == 'all' || $which == 'value') {
      $form['value'] = array(
        '#type' => 'textfield',
        '#title' => t('Value'),
        '#size' => 60,
        '#default_value' => $this->value,
      );
      if (!empty($form_state['exposed']) && !isset($form_state['input'][$identifier])) {
        $form_state['input'][$identifier] = $this->value;
      }

      if ($which == 'all') {
        $form['value'] += array(
          '#process' => array('ctools_dependent_process'),
          '#dependency' => array($source => $this->operator_values(1)),
        );
      }
    }

    if (!isset($form['value'])) {
      // Ensure there is something in the 'value'.
      $form['value'] = array(
        '#type' => 'value',
        '#value' => NULL
      );
    }
    // Attach the SPARQL specific form elements.
    $form = _sparql_views_attach_form_elements($form, $this->options, $this->selected_sparql_options);
  }

  function query() {
    $field_name = $this->definition['base field'];
    $info = $this->operators();
    if (!empty($info[$this->operator]['method'])) {
      $this->{$info[$this->operator]['method']}($field_name);
    }
  }
  
  function op_equal() {
    $value_type = $this->options['sparql_options']['value_type'];
    $language = $this->options['sparql_options']['language'];
    if ($value_type != 'number') {
      $object = $this->query->format_replacement($this->value, $value_type, $language);
    }
    else {
      $var = '?var' . rand();
      $this->query->add_filter('=', array($var, $this->value));
      $object = $var;
    }
    $relationship = $this->options['relationship'];

    if ($relationship == 'none') {
      $subject = $this->definition['subject'];
    }
    else {
      $subject = $relationship;
    }
    $predicate = $this->definition['rdf_mapping'][0];
    $this->query->add_triple($subject, $predicate, $object);
  }

  function op_not_equal($field_name) {
    $value_type = $this->options['sparql_options']['value_type'];
    $language = $this->options['sparql_options']['language'];

    $object = '?var' . rand();
    $values = preg_split('/,\s/', $this->value);
    foreach ($values as $value) {
      $val = $this->query->format_replacement($value, $value_type, $language);
      $this->query->filters[] = '(STR(' . $object . ') != "' . $val . '")';
    }

    $relationship = $this->options['relationship'];

    if ($relationship == 'none') {
      $subject = $this->definition['subject'];
    }
    else {
      $subject = $relationship;
    }
    $predicate = $this->definition['rdf_mapping'][0];
    $this->query->add_triple($subject, $predicate, $object);
  }

  function op_not_bound($field_name) {
    $relationship = $this->options['relationship'];
    if ($relationship == 'none') {
      $subject = $this->definition['subject'];
    }
    else {
      $subject = $relationship;
    }
    $this->query->add_filter('!BOUND', array($subject));
  }

  function op_is_uri($field_name) {
    $this->query->add_filter('isURI', array('?' . $field_name));
  }

  function op_is_of_datatype($field_name) {
    $value = $this->query->format_replacement($this->options['value'], 'uri');

    $relationship = $this->options['relationship'];

    if ($relationship == 'none') {
      $subject = $this->definition['base field'];
    }
    else {
      $subject = $relationship;
    }

    $this->query->add_filter('=', array('DATATYPE(?' . $subject .')', '"' . $value . '"'));
  }

}
